class FormNotInstantiated(Exception):
    pass

class FieldNotInstantiated(Exception):
    pass

class FieldNotFound(Exception):
    pass

class FormNotFound(Exception):
    pass

class ValidationError(Exception):
    pass
