QDialog,
QWidget[form='true'] {
    border: 0;
    margin: 0;
    padding: 0;
}

QWidget[header='true'] {
    border: 0;
    margin: 0;
    padding: 0;
    background: rgb(45, 45, 45);
}

QLabel[title='true'] {
    font: 14pt;
}

QLabel[description='true'] {
    font: 10pt;
}

QLabel{
    font: 10pt;
}

QLabel[valid='false']{
    color: rgb(255, 50, 50);
}

QLabel[err='true']{
    font: 10pt;
    color: rgb(255, 50, 50);
}

QLabel[clickable='true']{
    padding-bottom: 4;
}

QLabel[clickable='true']:hover{
    color: #CCC;
}

QPushButton{
    height: 24px;
    padding: 0 10 0 10;
}
QPushButton::menu-indicator{width:0px;}

QPushButton[grouptitle='true']{
    background: rgb(68, 68, 68);
    border: 0;
    border-top: 1px solid rgb(55, 55, 55);
    border-bottom: 1px solid rgb(55, 55, 55);
    margin: 0;
    padding: 0;
    padding-left: 20px;
    text-align: left;
}

QPushButton[grouptitle='true']:hover{
    background: rgb(78, 78, 78);
    border-radius: 0;
}

QPushButton[grouptitle='true']:pressed{}

QWidget[groupwidget='true']{
    background:  rgb(55, 55, 55);
}

QComboBox {
    padding-left: 13;
    height: 24px;
}

QLineEdit{
    height: 24px;
    padding-left: 10;
}


QSpinBox,
QDoubleSpinBox {
    background: #2B2B2B;
    border: 1px solid #505050;
    border-radius: 3;
    padding-left: 10;
    height: 26px;
}

QSpinBox:focus,
QDoubleSpinBox:focus {
    background: #2B2B2B;
    border: 2px solid #507893;
    padding-left: 10;
}

QSpinBox[valid='false'],
QDoubleSpinBox[valid='false'] {
    border: 1px solid rgb(203, 40, 40);
}


QSpinBox[valid='false']:focus,
QDoubleSpinBox[valid='false']:focus {
    border: 1px solid rgb(203, 40, 40);
}

QSpinBox::up-button,
QDoubleSpinBox::up-button {
    background: rgb(255, 255, 255, 0);
    border-bottom: 5px solid rgb(185,185,185);
    border-left: 5px solid rgb(255, 255, 255, 0);
    border-right: 5px solid rgb(255, 255, 255, 0);
    border-top: 0px solid rgb(255, 255, 255, 0);
    margin-bottom: 4px;
    margin-left: 2px;
    margin-right: 4px;
    margin-top: -2px;
    subcontrol-origin: border;
    subcontrol-position: top right;
}

QSpinBox::down-button,
QDoubleSpinBox::down-button {
    background: rgb(255, 255, 255, 0);
    border-bottom: 0px solid rgb(255, 255, 255, 0);
    border-left: 5px solid rgb(255, 255, 255, 0);
    border-right: 5px solid rgb(255, 255, 255, 0);
    border-top: 5px solid rgb(185,185,185);
    margin-bottom: -2px;
    margin-left: 2px;
    margin-right: 4px;
    margin-top: 4px;
    subcontrol-origin: border;
    subcontrol-position: bottom right;
}

QSpinBox::up-button:hover,
QDoubleSpinBox::up-button:hover {
    border-bottom: 5px solid rgb(235,235,235);
    border-left: 5px solid rgb(255, 255, 255, 0);
    border-right: 5px solid rgb(255, 255, 255, 0);
    border-top: 0px solid rgb(255, 255, 255, 0);
}

QSpinBox::down-button:hover,
QDoubleSpinBox::down-button:hover {
    border-bottom: 0px solid rgb(255, 255, 255, 0);
    border-left: 5px solid rgb(255, 255, 255, 0);
    border-right: 5px solid rgb(255, 255, 255, 0);
    border-top: 5px solid rgb(235,235,235);
}

QSpinBox::up-button:pressed,
QDoubleSpinBox::up-button:pressed {
    border-bottom: 5px solid #FFF;
    border-left: 5px solid rgb(255, 255, 255, 0);
    border-right: 5px solid rgb(255, 255, 255, 0);
    border-top: 0px solid rgb(255, 255, 255, 0);
    margin-bottom: 3px;
}

QSpinBox::down-button:pressed,
QDoubleSpinBox::down-button:pressed {
    margin-top: 3px;
    border-bottom: 0px solid rgb(255, 255, 255, 0);
    border-left: 5px solid rgb(255, 255, 255, 0);
    border-right: 5px solid rgb(255, 255, 255, 0);
    border-top: 5px solid #FFF;
}

QSpinBox::up-button:disabled,
QDoubleSpinBox::up-button:disabled {
    background: rgb(55, 55, 55);
}

QSpinBox::down-button:disabled,
QDoubleSpinBox::down-button:disabled {
    background: rgb(55, 55, 55);
}
